package operator

import (
	"context"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
	"go.mongodb.org/mongo-driver/mongo"
	"go.mongodb.org/mongo-driver/mongo/options"
)

type WorkerInstalledInfo struct {
	ID       string `bson:"_id,omitempty" json:"id"`
	WorkerId string `bson:"worker_id" json:"worker_id"` // use worker MinerPubkey
	ModelId  int    `bson:"model_id" json:"model_id"`   // installed model.
	GpuFree  int64  `bson:"gpu_free" json:"gpu_free"`
	GpuSeq   int    `bson:"gpu_seq" json:"gpu_seq"`
}

type WorkerInstalledOperator struct {
	client *mongo.Client
	col    *mongo.Collection
}

func NewDBWorkerInstalled(client *mongo.Client, database string) *WorkerInstalledOperator {
	return &WorkerInstalledOperator{
		client: client,
		col:    client.Database(database).Collection("worker_installed_info"),
	}
}
func (d WorkerInstalledOperator) CreateIndex(ctx context.Context) error {
	_, err := d.col.Indexes().CreateOne(ctx, mongo.IndexModel{
		Keys: bson.D{{"model_id", 1}, {"gpu_free", 1}},
		//Options: options.Index().SetUnique(true),
	})
	return err
}

func (d *WorkerInstalledOperator) InsertMany(ctx context.Context, workers []*WorkerInstalledInfo) (*mongo.InsertManyResult, error) {
	var docs []interface{}
	for _, worker := range workers {
		docs = append(docs, worker)
	}
	return d.col.InsertMany(ctx, docs)
}

func (d *WorkerInstalledOperator) DeleteMany(ctx context.Context, workerid string, models []int) (int, error) {
	res, err := d.col.DeleteMany(ctx, bson.M{"worker_id": workerid, "model_id": bson.M{"$in": models}})
	return int(res.DeletedCount), err
}

func (d *WorkerInstalledOperator) Insert(ctx context.Context, worker *WorkerInstalledInfo) (*mongo.InsertOneResult, error) {
	return d.col.InsertOne(ctx, worker)
}

func (d *WorkerInstalledOperator) UpdateGpuFree(ctx context.Context, workerid string, gpuFree int64, gpuSeq int) error {
	update := bson.M{"$set": bson.M{"gpu_free": gpuFree, "gpu_seq": gpuSeq}}
	_, err := d.col.UpdateMany(ctx, bson.M{"worker_id": workerid}, update)
	return err
}

func (d *WorkerInstalledOperator) DeleteByWorkerId(ctx context.Context, workerid string) (int, error) {
	res, err := d.col.DeleteMany(ctx, bson.M{"worker_id": workerid})
	return int(res.DeletedCount), err
}

func (d *WorkerInstalledOperator) Get(ctx context.Context, id string) (*WorkerInstalledInfo, error) {
	var worker WorkerInstalledInfo
	oid, _ := primitive.ObjectIDFromHex(id)
	err := d.col.FindOne(ctx, bson.M{"_id": oid}).Decode(&worker)
	return &worker, err
}

func (d *WorkerInstalledOperator) Clear() int64 {
	res, _ := d.col.DeleteMany(context.Background(), bson.M{})
	if res != nil {
		return res.DeletedCount
	}
	return 0
}

//
//func (d *WorkerInstalledOperator) FindWorkerByModelId(ctx context.Context, modelId int, limit int) ([]*WorkerInstalledInfo, error) {
//	// find all worker that at least one installed model's mode_id is equal modelId
//	// sort by gpu_mem_free.
//	findOptions := options.Find()
//	findOptions.SetLimit(int64(limit))
//	findOptions.SetSort(bson.D{{"gpu_free", -1}})
//
//	selector := bson.M{"model_id": modelId}
//	cursor, err := d.col.Find(ctx, selector, findOptions)
//	if err != nil {
//		return nil, err
//	}
//	defer cursor.Close(ctx)
//
//	var workers []*WorkerInstalledInfo
//	if err = cursor.All(ctx, &workers); err != nil {
//		return nil, err
//	}
//	return workers, nil
//}

func (d *WorkerInstalledOperator) FindWorkerByModelIdAndGpuMem(ctx context.Context, modelId int, mem int64, limit int) ([]*WorkerInstalledInfo, error) {
	// find all worker that at least one installed model's mode_id is equal modelId
	// sort by gpu_free
	findOptions := options.Find()
	findOptions.SetLimit(int64(limit))
	findOptions.SetSort(bson.D{{"gpu_free", -1}})

	selector := bson.M{"model_id": modelId, "gpu_free": bson.M{"$gte": mem}}
	cursor, err := d.col.Find(ctx, selector, findOptions)
	if err != nil {
		return nil, err
	}
	defer cursor.Close(ctx)

	var workers []*WorkerInstalledInfo
	if err = cursor.All(ctx, &workers); err != nil {
		return nil, err
	}
	return workers, nil
}
