package operator

import (
	"context"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
	"go.mongodb.org/mongo-driver/mongo"
	"go.mongodb.org/mongo-driver/mongo/options"
)

type WorkerRunningInfo struct {
	ID       string `bson:"_id,omitempty" json:"id"`
	WorkerId string `bson:"worker_id" json:"worker_id"` // use worker MinerPubkey
	ModelId  int    `bson:"model_id" json:"model_id"`
	ExecTime int    `bson:"exec_time" json:"exec_time"`
}

type WorkerRunningOperator struct {
	client *mongo.Client
	col    *mongo.Collection
}

func NewDBWorkerRunning(client *mongo.Client, database string) *WorkerRunningOperator {
	return &WorkerRunningOperator{
		client: client,
		col:    client.Database(database).Collection("worker_running_info"),
	}
}

func (d *WorkerRunningOperator) CreateIndex(ctx context.Context) error {
	_, err := d.col.Indexes().CreateOne(ctx, mongo.IndexModel{
		Keys: bson.D{{"model_id", 1}, {"exec_time", 1}},
		//Options: options.Index().SetUnique(true),
	})
	return err
}

func (d *WorkerRunningOperator) Clear() int64 {
	result, _ := d.col.DeleteMany(context.Background(), bson.M{})
	if result != nil {
		return result.DeletedCount
	}
	return 0
}

func (d *WorkerRunningOperator) InsertMany(ctx context.Context, workers []*WorkerRunningInfo) (*mongo.InsertManyResult, error) {
	var docs []interface{}
	for _, worker := range workers {
		docs = append(docs, worker)
	}
	return d.col.InsertMany(ctx, docs)
}

func (d *WorkerRunningOperator) DeleteMany(ctx context.Context, workerid string, models []int) (int, error) {
	res, err := d.col.DeleteMany(ctx, bson.M{"worker_id": workerid, "model_id": bson.M{"$in": models}})
	return int(res.DeletedCount), err
}

func (d *WorkerRunningOperator) Insert(ctx context.Context, worker *WorkerRunningInfo) (*mongo.InsertOneResult, error) {
	return d.col.InsertOne(ctx, worker)
}

func (d *WorkerRunningOperator) UpdateExecTime(ctx context.Context, workerid string, model int, execTime int) error {
	filter := bson.M{"worker_id": workerid, "model_id": model}
	update := bson.M{"$set": bson.M{"exec_time": execTime}}
	_, err := d.col.UpdateOne(ctx, filter, update)
	return err
}

func (d *WorkerRunningOperator) Get(ctx context.Context, id string) (*WorkerRunningInfo, error) {
	var worker WorkerRunningInfo
	oid, _ := primitive.ObjectIDFromHex(id)
	err := d.col.FindOne(ctx, bson.M{"_id": oid}).Decode(&worker)
	return &worker, err
}

func (d *WorkerRunningOperator) DeleteByWorkerId(ctx context.Context, workerid string) (int, error) {
	res, err := d.col.DeleteMany(ctx, bson.M{"worker_id": workerid})
	return int(res.DeletedCount), err
}

func (d *WorkerRunningOperator) FindWorkerByModelId(ctx context.Context, modelId int, limit int) ([]*WorkerRunningInfo, error) {
	// find all worker that at least one running model's mode_id is equal modelId
	// sort by wait time
	findOptions := options.Find()
	findOptions.SetLimit(int64(limit))
	findOptions.SetSort(bson.D{{"exec_time", -1}})

	selector := bson.M{"model_id": modelId}
	cursor, err := d.col.Find(ctx, selector, findOptions)
	if err != nil {
		return nil, err
	}
	defer cursor.Close(ctx)

	var workers []*WorkerRunningInfo
	if err = cursor.All(ctx, &workers); err != nil {
		return nil, err
	}
	return workers, nil
}
