package operator

import (
	"context"
	"fmt"
	"github.com/google/uuid"
	"go.mongodb.org/mongo-driver/bson/primitive"
	"log"
	"testing"
)

func TestWorkerRunningOperator_Insert(t *testing.T) {
	client, err := ConnectMongoDB()
	if err != nil {
		log.Fatal(err)
	}
	db := NewDBWorkerRunning(client, database)
	defer db.client.Disconnect(context.Background())
	w := &WorkerRunningInfo{
		WorkerId: uuid.NewString(),
		ModelId:  1,
		ExecTime: 100,
	}
	is, err := db.Insert(context.Background(), w)
	if err != nil {
		t.Errorf("insert worker failed with err:%s", err)
	}
	id := is.InsertedID.(primitive.ObjectID)

	worker, _ := db.Get(context.Background(), id.Hex())
	if worker.WorkerId != w.WorkerId {
		t.Errorf("insert worker failed with err:%s", err)
	}
}

func TestWorkerRunningOperator_InsertMany(t *testing.T) {
	client, err := ConnectMongoDB()
	if err != nil {
		log.Fatal(err)
	}
	db := NewDBWorkerRunning(client, database)
	defer db.client.Disconnect(context.Background())
	ws := make([]*WorkerRunningInfo, 0)
	for i := 0; i < 10; i++ {
		w := &WorkerRunningInfo{
			WorkerId: uuid.NewString(),
			ModelId:  i,
			ExecTime: 100,
		}
		ws = append(ws, w)
	}
	if res, err := db.InsertMany(context.Background(), ws); err != nil {
		t.Errorf("insert worker failed with err:%s", err)
	} else {
		if len(res.InsertedIDs) != len(ws) {
			t.Errorf("insert worker failed with err:%s", err)
		}
	}
}

func TestWorkerRunningOperator_DeleteMany(t *testing.T) {
	client, err := ConnectMongoDB()
	if err != nil {
		log.Fatal(err)
	}
	db := NewDBWorkerRunning(client, database)
	defer db.client.Disconnect(context.Background())
	ws := make([]*WorkerRunningInfo, 0)
	wid := uuid.NewString()
	for i := 0; i < 10; i++ {
		w := &WorkerRunningInfo{
			WorkerId: wid,
			ModelId:  i,
			ExecTime: 100,
		}
		ws = append(ws, w)
	}
	if _, err := db.InsertMany(context.Background(), ws); err != nil {
		t.Errorf("insert worker failed with err:%s", err)
	}
	deleteId := []int{1, 2, 3}
	if n, err := db.DeleteMany(context.Background(), wid, deleteId); err != nil {
		t.Errorf("delete worker failed with err:%s", err)
	} else {
		if n != len(deleteId) {
			t.Errorf("delete worker failed with err:%s", err)
		}
	}
}

func TestWorkerRunningOperator_DeleteByWorkerId(t *testing.T) {
	client, err := ConnectMongoDB()
	if err != nil {
		log.Fatal(err)
	}
	db := NewDBWorkerRunning(client, database)
	defer db.client.Disconnect(context.Background())
	ws := make([]*WorkerRunningInfo, 0)
	wid := uuid.NewString()
	for i := 0; i < 10; i++ {
		w := &WorkerRunningInfo{
			WorkerId: wid,
			ModelId:  i,
			ExecTime: 100,
		}
		ws = append(ws, w)
	}
	if _, err := db.InsertMany(context.Background(), ws); err != nil {
		t.Errorf("insert worker failed with err:%s", err)
	}
	if n, err := db.DeleteByWorkerId(context.Background(), wid); err != nil {
		t.Errorf("delete worker failed with err:%s", err)
	} else {
		if n != len(ws) {
			t.Errorf("delete worker failed with err:%s", err)
		}
	}
}

func TestWorkerRunningOperator_UpdateExecTime(t *testing.T) {
	client, err := ConnectMongoDB()
	if err != nil {
		log.Fatal(err)
	}
	db := NewDBWorkerRunning(client, database)
	defer db.client.Disconnect(context.Background())
	w := &WorkerRunningInfo{
		WorkerId: uuid.NewString(),
		ModelId:  1,
		ExecTime: 100,
	}
	is, err := db.Insert(context.Background(), w)
	if err != nil {
		t.Errorf("insert worker failed with err:%s", err)
	}

	if err := db.UpdateExecTime(context.Background(), w.WorkerId, w.ModelId, 200); err != nil {
		t.Errorf("update worker failed with err:%s", err)
	}
	id := is.InsertedID.(primitive.ObjectID)
	info, err := db.Get(context.Background(), id.Hex())
	if err != nil {
		t.Errorf("update worker failed with err:%s", err)
	}
	if info.ExecTime != 200 {
		t.Errorf("update worker failed with err:%s", err)
	}
}

func BenchmarkDbWorkerRunning_FindWorkerByModelId(b *testing.B) {
	client, err := ConnectMongoDB()
	if err != nil {
		log.Fatal(err)
	}
	b.StopTimer()
	db := NewDBWorkerRunning(client, database)
	defer db.client.Disconnect(context.Background())
	once.Do(func() {
		db.Clear()
		if err := db.CreateIndex(context.Background()); err != nil {
			b.Fatal(err)
		}
		idlist = initdata(client, 10000, true, false, true)
	})
	b.StartTimer()
	b.ResetTimer()
	for i := 0; i < b.N; i++ {
		modelId := getRandIdInt(maxModelId)
		if w, err := db.FindWorkerByModelId(context.Background(), modelId, 10); err != nil {
			panic(fmt.Sprintf("find worker failed with err:%s", err))
		} else if len(w) == 0 {
			b.Logf("FindWorkerByModelId find %d with id %d\n", len(w), modelId)
		}
	}
}

func BenchmarkDbWorkerRunning_FindWorkerByModelId_Parallel(b *testing.B) {
	client, err := ConnectMongoDB()
	if err != nil {
		log.Fatal(err)
	}
	b.StopTimer()
	db := NewDBWorkerRunning(client, database)
	defer db.client.Disconnect(context.Background())
	once.Do(func() {
		db.Clear()
		if err := db.CreateIndex(context.Background()); err != nil {
			b.Fatal(err)
		}
		idlist = initdata(client, 10000, true, false, true)
	})
	b.StartTimer()
	b.ResetTimer()
	b.RunParallel(func(pb *testing.PB) {
		for pb.Next() {
			modelId := getRandIdInt(maxModelId)
			if w, err := db.FindWorkerByModelId(context.Background(), modelId, 10); err != nil {
				panic(fmt.Sprintf("find worker failed with err:%s", err))
			} else if len(w) == 0 {
				b.Logf("FindWorkerByModelId find %d with id %d\n", len(w), modelId)
			}
		}
	})
}
