package operator

import (
	"context"
	"github.com/odysseus/mogo/types"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
	"go.mongodb.org/mongo-driver/mongo"
	"go.mongodb.org/mongo-driver/mongo/options"
)

type WorkerInfo struct {
	ID       string              `bson:"_id,omitempty" json:"id"`
	WorkerId string              `bson:"worker_id" json:"worker_id"` // use worker MinerPubkey
	NodeInfo *types.NodeInfo     `bson:"node_info" json:"node_info"`
	Models   *types.ModelInfo    `bson:"model_infos" json:"model_infos"`
	Hardware *types.HardwareInfo `bson:"hardware" json:"hardware"`
}

type WorkerInfoOperator struct {
	client *mongo.Client
	col    *mongo.Collection
}

func NewDBWorker(client *mongo.Client, database string) *WorkerInfoOperator {
	return &WorkerInfoOperator{
		client: client,
		col:    client.Database(database).Collection("workerinfo"),
	}
}

func (d *WorkerInfoOperator) CreateIndex(ctx context.Context) error {
	_, err := d.col.Indexes().CreateMany(ctx, []mongo.IndexModel{
		{
			Keys: bson.D{{"worker_id", 1}},
		},
		{
			Keys: bson.D{{"model_infos.running_models.wait_time", 1},
				{"model_infos.running_models.model_id", 1}},
		},
	})
	return err
}

func (d *WorkerInfoOperator) Clear() int64 {
	res, _ := d.col.DeleteMany(context.Background(), bson.M{})
	if res != nil {
		return res.DeletedCount
	}
	return 0
}

func (d *WorkerInfoOperator) Get(ctx context.Context, id string) (*WorkerInfo, error) {
	var worker WorkerInfo
	oid, _ := primitive.ObjectIDFromHex(id)
	err := d.col.FindOne(ctx, bson.M{"_id": oid}).Decode(&worker)
	return &worker, err
}

func (d *WorkerInfoOperator) GetAllWorkerId(ctx context.Context) ([]string, error) {
	filter := bson.D{}
	opts := options.Find().SetProjection(bson.D{{"worker_id", 1}})

	cursor, err := d.col.Find(ctx, filter, opts)
	if err != nil {
		return nil, err
	}
	defer cursor.Close(ctx)
	var results []WorkerInfo
	if err = cursor.All(context.TODO(), &results); err != nil {
		return nil, err
	}
	var workerIds []string
	for _, result := range results {
		workerIds = append(workerIds, result.WorkerId)
	}
	return workerIds, err
}

func (d *WorkerInfoOperator) InsertWorker(ctx context.Context, worker *WorkerInfo) (*mongo.InsertOneResult, error) {
	return d.col.InsertOne(ctx, worker)
}

func (d *WorkerInfoOperator) InsertMany(ctx context.Context, workers []*WorkerInfo) (*mongo.InsertManyResult, error) {
	var docs []interface{}
	for _, worker := range workers {
		docs = append(docs, worker)
	}
	return d.col.InsertMany(ctx, docs)
}

func (d *WorkerInfoOperator) DeleteByWorkerId(ctx context.Context, workerid string) (int, error) {
	res, err := d.col.DeleteMany(ctx, bson.M{"worker_id": workerid})
	return int(res.DeletedCount), err
}

//func (d *WorkerInfoOperator) UpdateModel(ctx context.Context, workerid string, models *types.ModelInfo) error {
//	update := bson.M{"$set": bson.M{"model_infos": models}}
//	_, err := d.col.UpdateOne(ctx, bson.M{"worker_id": workerid}, update)
//	return err
//}

func (d *WorkerInfoOperator) UpdateBenefitAddr(ctx context.Context, workerid string, benefitAddr string) error {
	update := bson.M{"$set": bson.M{"node_info.benefit_addr": benefitAddr}}
	_, err := d.col.UpdateOne(ctx, bson.M{"worker_id": workerid}, update)
	return err
}

func (d *WorkerInfoOperator) AddRunningModels(ctx context.Context, workerid string, models []*types.RunningModel) error {
	updates := make([]mongo.WriteModel, len(models))
	for i, model := range models {
		updates[i] = mongo.NewUpdateOneModel().SetFilter(bson.M{"worker_id": workerid}).SetUpdate(bson.M{"$push": bson.M{"model_infos.running_models": model}})
	}
	_, err := d.col.BulkWrite(ctx, updates)
	return err
}

func (d *WorkerInfoOperator) DeleteRunningModels(ctx context.Context, workerid string, models []int) error {
	updates := make([]mongo.WriteModel, len(models))
	for i, model := range models {
		updates[i] = mongo.NewUpdateOneModel().SetFilter(bson.M{"worker_id": workerid}).SetUpdate(bson.M{"$pull": bson.M{"model_infos.running_models": bson.M{"model_id": model}}})
	}
	_, err := d.col.BulkWrite(ctx, updates)
	return err
}

func (d *WorkerInfoOperator) AddInstalledModels(ctx context.Context, workerid string, models []*types.InstalledModel) error {
	updates := make([]mongo.WriteModel, len(models))
	for i, model := range models {
		updates[i] = mongo.NewUpdateOneModel().SetFilter(bson.M{"worker_id": workerid}).SetUpdate(bson.M{"$push": bson.M{"model_infos.installed_models": model}})
	}
	_, err := d.col.BulkWrite(ctx, updates)
	return err
}

func (d *WorkerInfoOperator) DeleteInstalledModels(ctx context.Context, workerid string, modelids []int) error {
	updates := make([]mongo.WriteModel, len(modelids))
	for i, modelid := range modelids {
		updates[i] = mongo.NewUpdateOneModel().SetFilter(bson.M{"worker_id": workerid}).SetUpdate(bson.M{"$pull": bson.M{"model_infos.installed_models": bson.M{"model_id": modelid}}})
	}
	_, err := d.col.BulkWrite(ctx, updates)
	return err
}

func (d *WorkerInfoOperator) UpdateInstalledModelStatus(ctx context.Context, workerid string, modelid int, lastRunTime int64) error {
	update := bson.M{"$set": bson.M{"model_infos.installed_models.$.last_run_time": lastRunTime}}
	_, err := d.col.UpdateOne(ctx, bson.M{"worker_id": workerid, "model_infos.installed_models.model_id": modelid}, update)
	return err
}

func (d *WorkerInfoOperator) UpdateRunningModelStatus(ctx context.Context, workerid string,
	modelid int, lastWorkTime int64, totalRunCount int64, exectime int) error {
	update := bson.M{"$set": bson.M{"model_infos.running_models.$.last_work_time": lastWorkTime, "model_infos.running_models.$.total_run_count": totalRunCount,
		"model_infos.running_models.$.exec_time": exectime}}
	_, err := d.col.UpdateOne(ctx, bson.M{"worker_id": workerid, "model_infos.running_models.model_id": modelid}, update)
	return err
}

//
//func (d *WorkerInfoOperator) UpdateRunningModelInfo(ctx context.Context, workerid string, modelid string, execTime int) error {
//	// update running model exec time.
//	update := bson.M{"$set": bson.M{"model_infos.running_models.$.exec_time": execTime}}
//	_, err := d.col.UpdateOne(ctx, bson.M{"worker_id": workerid, "model_infos.running_models.model_id": modelid}, update)
//	return err
//}
//
//func (d *WorkerInfoOperator) UpdateInstalledModelInfo(ctx context.Context, workerid string, modelid string, gpuFree int64) error {
//	// update installed model gpu free.
//	update := bson.M{"$set": bson.M{"model_infos.installed_models.$.gpu_free": gpuFree}}
//	_, err := d.col.UpdateOne(ctx, bson.M{"worker_id": workerid, "model_infos.installed_models.model_id": modelid}, update)
//	return err
//}

func (d *WorkerInfoOperator) UpdateHardware(ctx context.Context, workerid string, hardware *types.HardwareInfo) error {
	update := bson.M{"$set": bson.M{"hardware": hardware}}
	_, err := d.col.UpdateOne(ctx, bson.M{"worker_id": workerid}, update)
	return err
}

//func (d *WorkerInfoOperator) UpdateNodeInfo(ctx context.Context, workerid string, nodeInfo *types.NodeInfo) error {
//	update := bson.M{"$set": bson.M{"node_info": nodeInfo}}
//	_, err := d.col.UpdateOne(ctx, bson.M{"worker_id": workerid}, update)
//	return err
//}

func (d *WorkerInfoOperator) UpdateWorker(ctx context.Context, worker *WorkerInfo) error {
	_, err := d.col.ReplaceOne(ctx, bson.M{"worker_id": worker.WorkerId}, worker)
	return err
}

func (d *WorkerInfoOperator) UpdateHardwareUsage(ctx context.Context, workerid string, usage types.DeviceUsage) error {
	_, err := d.col.BulkWrite(ctx, []mongo.WriteModel{
		mongo.NewUpdateOneModel().SetFilter(bson.M{"worker_id": workerid}).SetUpdate(bson.M{"$set": bson.M{"hardware.CPU.usage": usage.CpuUsage}}),
		mongo.NewUpdateOneModel().SetFilter(bson.M{"worker_id": workerid}).SetUpdate(bson.M{"$set": bson.M{"hardware.RAM.free": usage.RamUsage}}),
		mongo.NewUpdateOneModel().SetFilter(bson.M{"worker_id": workerid}).SetUpdate(bson.M{"$set": bson.M{"hardware.DISK.free": usage.DiskUsage}}),
		mongo.NewUpdateOneModel().SetFilter(bson.M{"worker_id": workerid}).SetUpdate(bson.M{"$set": bson.M{"hardware.NET.bandwidth": usage.NetBandwidth}}),
	})
	return err
}

func (d *WorkerInfoOperator) UpdateGPUUsage(ctx context.Context, workerid string, usages []types.GpuUsage) error {
	updates := make([]mongo.WriteModel, len(usages))
	for i, usage := range usages {
		// update gpu info
		updates[i] = mongo.NewUpdateOneModel().SetFilter(bson.M{"worker_id": workerid, "hardware.GPU.seq": usage.GpuSeq}).SetUpdate(bson.M{"$set": bson.M{"hardware.GPU.$.usage": usage.Usage,
			"hardware.GPU.$.mem_free": usage.MemFree, "hardware.GPU.$.temp": usage.Temp, "hardware.GPU.$.power_rt": usage.Powrt}})
	}
	_, err := d.col.BulkWrite(ctx, updates)
	return err
}

type WorkerFindResult struct {
	WorkerId string `bson:"worker_id" json:"worker_id"`
}

func (d *WorkerInfoOperator) FindWorkerByRunningModelAndSortByWaitTime(ctx context.Context, modelId string, limit int) ([]*WorkerFindResult, error) {
	// find all worker that at least one running model's mode_id is equal modelId
	// sort by wait time
	findOptions := options.Find()
	findOptions.SetLimit(int64(limit))
	findOptions.SetSort(bson.D{{"model_infos.running_models.wait_time", -1}})

	selector := bson.M{"model_infos.running_models.model_id": modelId}
	cursor, err := d.col.Find(ctx, selector, findOptions)
	if err != nil {
		return nil, err
	}
	defer cursor.Close(ctx)

	var find []*WorkerFindResult
	if err = cursor.All(ctx, &find); err != nil {
		return nil, err
	}
	return find, nil
}

func (d *WorkerInfoOperator) FindWorkerByInstallModelAndSortByGpuRam(ctx context.Context, modelId string, performance int, ram int64, limit int) ([]*WorkerFindResult, error) {
	// find all worker that at least one installed model's mode_id is equal modelId
	// sort by gpu ram
	findOptions := options.Find()
	findOptions.SetLimit(int64(limit))
	findOptions.SetSort(bson.D{{"hardware.GPU.mem_free", -1}})

	selector := bson.M{"model_infos.installed_models.model_id": modelId, "hardware.GPU.performance": bson.M{"$gte": performance}, "hardware.GPU.mem_free": bson.M{"$gte": ram}}
	cursor, err := d.col.Find(ctx, selector, findOptions)
	if err != nil {
		return nil, err
	}
	defer cursor.Close(ctx)

	var find []*WorkerFindResult
	if err = cursor.All(ctx, &find); err != nil {
		return nil, err
	}
	return find, nil
}

func (d *WorkerInfoOperator) FindWorkerByWorkerId(ctx context.Context, workerId string) (*WorkerInfo, error) {
	var worker *WorkerInfo
	err := d.col.FindOne(ctx, bson.M{"worker_id": workerId}).Decode(&worker)
	return worker, err
}
