package operator

import (
	"context"
	crand "crypto/rand"
	"encoding/hex"
	"fmt"
	"github.com/google/uuid"
	"github.com/odysseus/mogo/types"
	"go.mongodb.org/mongo-driver/bson/primitive"
	"go.mongodb.org/mongo-driver/mongo"
	"go.mongodb.org/mongo-driver/mongo/options"
	"log"
	"math/rand"
	"strconv"
	"sync"
	"testing"
	"time"
)

var (
	maxModelId   = 1000
	idlist       = make([]string, 0, 1000000)
	workeridList = make([]string, 0, 1000000)
	database     = "test"
	once         = sync.Once{}
)

func ConnectMongoDB() (*mongo.Client, error) {
	uri := "mongodb://localhost:27017"
	username := "admin"
	passwd := "admin"
	ctx, _ := context.WithTimeout(context.Background(), 10*time.Second)
	client, err := mongo.Connect(ctx, options.Client().ApplyURI(uri).SetAuth(options.Credential{
		Username: username,
		Password: passwd,
	}))
	if err != nil {
		return nil, err
	}

	return client, nil
}

func initdata(client *mongo.Client, count int, running bool, installed bool, workerid bool) []string {
	t1 := time.Now()
	db := NewDBWorker(client, database)
	dbRunning := NewDBWorkerRunning(client, database)
	dbInstalled := NewDBWorkerInstalled(client, database)

	ids := make([]string, 0, count)

	// Insert 1,000,000 DbWorkerInfo to operator
	for i := 0; i < count; i++ {
		worker := generateAWorker()
		if workerid {
			ids = append(ids, worker.WorkerId)
		}
		result, err := db.InsertWorker(context.Background(), worker)
		if err != nil {
			panic(fmt.Sprintf("insert worker failed with err:%s", err))
		}
		if running {
			// add worker running info to dbRunning
			runnings := make([]*WorkerRunningInfo, 0, len(worker.Models.RunningModels))
			for _, model := range worker.Models.RunningModels {
				id, _ := strconv.Atoi(model.ModelID)
				runnings = append(runnings, &WorkerRunningInfo{
					WorkerId: worker.WorkerId,
					ModelId:  id,
					ExecTime: model.ExecTime,
				})
			}
			_, err = dbRunning.InsertMany(context.Background(), runnings)
			if err != nil {
				panic(fmt.Sprintf("insert worker running failed with err:%s", err))
			}
		}
		if installed {
			// add worker installed info to dbInstalled
			installeds := make([]*WorkerInstalledInfo, 0, len(worker.Models.InstalledModels))
			for _, model := range worker.Models.InstalledModels {
				id, _ := strconv.Atoi(model.ModelID)
				installeds = append(installeds, &WorkerInstalledInfo{
					WorkerId: worker.WorkerId,
					ModelId:  id,
					GpuFree:  1024 * 1024 * 1024,
					GpuSeq:   0,
				})
			}
			_, err = dbInstalled.InsertMany(context.Background(), installeds)
			if err != nil {
				panic(fmt.Sprintf("insert worker installed failed with err:%s", err))
			}
		}

		if !workerid {
			id, ok := result.InsertedID.(primitive.ObjectID)
			if !ok {
				panic("inserted id is not primitive.ObjectID")
			}
			ids = append(ids, id.Hex())
		}
		//fmt.Printf("insert worker %s: %v\n", id.Hex(), worker)
	}
	t2 := time.Now()
	fmt.Printf("init data cost %s\n", t2.Sub(t1).String())
	return ids
}

func getRandId(max int) string {
	return strconv.Itoa(rand.Intn(max) + 1)
}

func getRandIdInt(max int) int {
	return rand.Intn(max) + 1
}

func generateAWorker() *WorkerInfo {
	return &WorkerInfo{
		WorkerId: uuid.NewString(),
		NodeInfo: generateANodeInfo(),
		Models:   generateAModel(),
		Hardware: generateAHardware(),
	}
}

func generateANodeInfo() *types.NodeInfo {
	x := make([]byte, 32)
	y := make([]byte, 32)
	z := make([]byte, 20)
	crand.Read(x)
	crand.Read(y)
	crand.Read(z)

	return &types.NodeInfo{
		MinerPubkey:    hex.EncodeToString(x) + hex.EncodeToString(y),
		BenefitAddress: hex.EncodeToString(z),
	}
}

func generateACpu() *types.CpuInfo {
	return &types.CpuInfo{
		Model:   "Intel(R) Core(TM) i7-8700 CPU @ 3.20GHz",
		Number:  1,
		Cores:   8,
		Threads: 16,
		Usage:   int(rand.Intn(30) + 40),
	}
}

func generateADisk() *types.DiskInfo {
	return &types.DiskInfo{
		Total: 1024 * 1024 * 1024,
		Free:  100 * 1024 * 1024,
	}
}

func generateANet() *types.NetInfo {
	return &types.NetInfo{
		IP:        fmt.Sprintf("192.168.1.%d", rand.Intn(255)),
		Mac:       fmt.Sprintf("%02x:%02x:%02x:%02x:%02x:%02x", 1, 2, 3, 4, 5, 6),
		Bandwidth: rand.Intn(30) + 60,
	}
}

func generateARam() *types.RamInfo {
	return &types.RamInfo{
		Total: 32 * 1024 * 1024 * 1024,
		Free:  int64(rand.Intn(10) * 1024 * 1024 * 1024),
	}
}

func generateAGpuRam() int64 {
	return 1024 * 1024 * 1024 * int64(rand.Intn(3)*8+8) // 8, 16, 24
}

func generateAGpuModel() string {
	m := rand.Intn(4)*10 + 3060 // 3060 ~ 3090
	return fmt.Sprintf("Nvidia %d", m)
}

func generateAGpuPerformance() int {
	return rand.Intn(100) + 500
}

func generateAIdleGpu(seq int) *types.GpuInfo {
	ram := generateAGpuRam()
	return &types.GpuInfo{
		Seq:         seq,
		UUID:        uuid.NewString(),
		Model:       generateAGpuModel(),
		Performance: generateAGpuPerformance(),
		PowerRating: 100,
		MemTotal:    ram,
		MemFree:     ram,
		Usage:       0,
		Temp:        40,
		PowerRt:     30,
	}
}

func generateAUsageGpu(seq int) *types.GpuInfo {
	ram := generateAGpuRam()
	return &types.GpuInfo{
		Seq:         seq,
		UUID:        uuid.NewString(),
		Model:       generateAGpuModel(),
		Performance: generateAGpuPerformance(),
		PowerRating: 100,
		MemTotal:    ram,
		MemFree:     ram * int64(rand.Intn(3)+3) / 10,
		Usage:       rand.Intn(20) + 70,
		Temp:        40,
		PowerRt:     30,
	}
}

func generateAHardware() *types.HardwareInfo {
	return &types.HardwareInfo{
		CPU: generateACpu(),
		GPU: []*types.GpuInfo{
			generateAIdleGpu(0),
			generateAIdleGpu(1),
			generateAUsageGpu(2),
			generateAUsageGpu(3),
			generateAUsageGpu(4),
			generateAUsageGpu(5),
			generateAUsageGpu(6),
			generateAUsageGpu(7),
		},
		RAM:  generateARam(),
		DISK: generateADisk(),
		NET:  generateANet(),
	}
}

func generateAInstallModel() *types.InstalledModel {
	return &types.InstalledModel{
		ModelID:       getRandId(maxModelId),
		DiskSize:      101,
		InstalledTime: time.Now().Unix(),
		LastRunTime:   time.Now().Unix(),
	}
}

func generateAInstallModelWithId(id int) *types.InstalledModel {
	return &types.InstalledModel{
		ModelID:       strconv.Itoa(id),
		DiskSize:      101,
		InstalledTime: time.Now().Unix(),
		LastRunTime:   time.Now().Unix(),
	}
}

func generateARunningModel() *types.RunningModel {
	return &types.RunningModel{
		ModelID:       getRandId(maxModelId),
		GpuSeq:        rand.Intn(3),
		GpuRAM:        generateAGpuRam(),
		StartedTime:   time.Now().Unix(),
		LastWorkTime:  time.Now().Unix(),
		TotalRunCount: rand.Intn(100),
		ExecTime:      rand.Intn(100),
	}
}
func generateARunningModelWithId(id int) *types.RunningModel {
	return &types.RunningModel{
		ModelID:       strconv.Itoa(id),
		GpuSeq:        rand.Intn(3),
		GpuRAM:        generateAGpuRam(),
		StartedTime:   time.Now().Unix(),
		LastWorkTime:  time.Now().Unix(),
		TotalRunCount: rand.Intn(100),
		ExecTime:      rand.Intn(100),
	}
}

func generateAModel() *types.ModelInfo {
	m := &types.ModelInfo{
		InstalledModels: make([]*types.InstalledModel, 0, 1000),
		RunningModels:   make([]*types.RunningModel, 0, 1000),
	}
	for i := 0; i < 200; i++ {
		m.InstalledModels = append(m.InstalledModels, generateAInstallModel())
		if len(m.RunningModels) < 100 {
			m.RunningModels = append(m.RunningModels, generateARunningModel())
		}
	}
	return m
}

func TestWorkerInfoOperator_InsertWorker(t *testing.T) {
	client, err := ConnectMongoDB()
	if err != nil {
		log.Fatal(err)
	}
	db := NewDBWorker(client, database)
	defer db.client.Disconnect(context.Background())
	worker := generateAWorker()
	if _, err := db.InsertWorker(context.Background(), worker); err != nil {
		t.Errorf("insert worker failed with err:%s", err)
	}
}

func TestWorkerInfoOperator_GetAllWorkerId(t *testing.T) {
	client, err := ConnectMongoDB()
	if err != nil {
		log.Fatal(err)
	}
	db := NewDBWorker(client, database)
	defer db.client.Disconnect(context.Background())
	db.Clear()

	// insert first
	ws := make([]*WorkerInfo, 0)
	for i := 0; i < 10; i++ {
		w := generateAWorker()
		ws = append(ws, w)
	}
	if _, err := db.InsertMany(context.Background(), ws); err != nil {
		t.Errorf("insert worker failed with err:%s", err)
	}

	ids, err := db.GetAllWorkerId(context.Background())
	if err != nil {
		t.Errorf("get all worker failed with err:%s", err)
	}
	if len(ids) != len(ws) {
		t.Errorf("result check failed")
	}
}

func TestWorkerInfoOperator_InsertMany(t *testing.T) {
	client, err := ConnectMongoDB()
	if err != nil {
		log.Fatal(err)
	}
	db := NewDBWorker(client, database)
	defer db.client.Disconnect(context.Background())
	db.Clear()

	ws := make([]*WorkerInfo, 0)
	for i := 0; i < 10; i++ {
		w := generateAWorker()
		ws = append(ws, w)
	}
	if res, err := db.InsertMany(context.Background(), ws); err != nil {
		t.Errorf("insert worker failed with err:%s", err)
	} else {
		if len(res.InsertedIDs) != len(ws) {
			t.Errorf("insert worker failed with err:%s", err)
		}
	}
}

func TestWorkerInfoOperator_AddInstalledModels(t *testing.T) {
	client, err := ConnectMongoDB()
	if err != nil {
		log.Fatal(err)
	}
	db := NewDBWorker(client, database)
	defer db.client.Disconnect(context.Background())
	db.Clear()

	worker := &WorkerInfo{
		WorkerId: uuid.NewString(),
		NodeInfo: generateANodeInfo(),
		Models: &types.ModelInfo{
			InstalledModels: []*types.InstalledModel{
				&types.InstalledModel{
					ModelID:       "1",
					DiskSize:      101,
					InstalledTime: time.Now().Unix(),
					LastRunTime:   time.Now().Unix(),
				},
			},
			RunningModels: make([]*types.RunningModel, 0, 1000),
		},
		Hardware: generateAHardware(),
	}
	is, err := db.InsertWorker(context.Background(), worker)
	if err != nil {
		t.Errorf("insert worker failed with err:%s", err)
	}
	id := is.InsertedID.(primitive.ObjectID)
	newModel := &types.InstalledModel{
		ModelID:       "2",
		DiskSize:      101,
		InstalledTime: time.Now().Unix(),
		LastRunTime:   time.Now().Unix(),
	}
	if err := db.AddInstalledModels(context.Background(), worker.WorkerId, []*types.InstalledModel{newModel}); err != nil {
		t.Error("add installed models failed")
	}
	worker, err = db.Get(context.Background(), id.Hex())
	if err != nil {
		t.Error("get worker failed")
	}
	if len(worker.Models.InstalledModels) != 2 {
		t.Error("add installed models failed")

	}
}

func TestWorkerInfoOperator_UpdateGPUUsage(t *testing.T) {
	client, err := ConnectMongoDB()
	if err != nil {
		log.Fatal(err)
	}
	db := NewDBWorker(client, database)
	defer db.client.Disconnect(context.Background())
	db.Clear()
	worker := &WorkerInfo{
		WorkerId: uuid.NewString(),
		NodeInfo: generateANodeInfo(),
		Models: &types.ModelInfo{
			InstalledModels: make([]*types.InstalledModel, 0, 1000),
			RunningModels: []*types.RunningModel{
				&types.RunningModel{
					ModelID:       "1",
					GpuSeq:        0,
					GpuRAM:        1024 * 1024 * 1024,
					StartedTime:   time.Now().Unix(),
					LastWorkTime:  time.Now().Unix(),
					TotalRunCount: 100,
					ExecTime:      100,
				},
			},
		},
		Hardware: generateAHardware(),
	}
	_, err = db.InsertWorker(context.Background(), worker)
	if err != nil {
		t.Errorf("insert worker failed with err:%s", err)
	}

	gpuusages := []types.GpuUsage{
		{0, 50, 10, 20, 30},
		{1, 51, 11, 21, 31},
		{2, 52, 12, 22, 32},
	}

	// update gpu usage
	if err := db.UpdateGPUUsage(context.Background(), worker.WorkerId, gpuusages); err != nil {
		t.Error("update gpu usage failed")
	}
	b, err := db.FindWorkerByWorkerId(context.Background(), worker.WorkerId)
	if err != nil {
		t.Error("find worker failed")
	}
	if b.Hardware.GPU[0].Usage != gpuusages[0].Usage {
		t.Error("update gpu usage compare failed")
	}
}

func TestWorkerInfoOperator_AddRunningModels(t *testing.T) {
	client, err := ConnectMongoDB()
	if err != nil {
		log.Fatal(err)
	}
	db := NewDBWorker(client, database)
	defer db.client.Disconnect(context.Background())
	db.Clear()

	worker := &WorkerInfo{
		WorkerId: uuid.NewString(),
		NodeInfo: generateANodeInfo(),
		Models: &types.ModelInfo{
			InstalledModels: make([]*types.InstalledModel, 0, 1000),
			RunningModels: []*types.RunningModel{
				&types.RunningModel{
					ModelID:       "1",
					GpuSeq:        0,
					GpuRAM:        1024 * 1024 * 1024,
					StartedTime:   time.Now().Unix(),
					LastWorkTime:  time.Now().Unix(),
					TotalRunCount: 100,
					ExecTime:      100,
				},
			},
		},
		Hardware: generateAHardware(),
	}
	is, err := db.InsertWorker(context.Background(), worker)
	if err != nil {
		t.Errorf("insert worker failed with err:%s", err)
	}
	id := is.InsertedID.(primitive.ObjectID)
	newModel := &types.RunningModel{
		ModelID:       "2",
		GpuSeq:        1,
		GpuRAM:        1024 * 1024 * 1024,
		StartedTime:   time.Now().Unix(),
		LastWorkTime:  time.Now().Unix(),
		TotalRunCount: 100,
		ExecTime:      100,
	}
	if err := db.AddRunningModels(context.Background(), worker.WorkerId, []*types.RunningModel{newModel}); err != nil {
		t.Error("add running models failed")
	}
	worker, err = db.Get(context.Background(), id.Hex())
	if err != nil {
		t.Error("get worker failed")
	}
	if len(worker.Models.RunningModels) != 2 {
		t.Error("add running models failed")
	}
}

func BenchmarkGenerateWorker(b *testing.B) {
	for i := 0; i < b.N; i++ {
		generateAWorker()
	}
}

func BenchmarkDbWorker_InsertWorker(b *testing.B) {
	client, err := ConnectMongoDB()
	if err != nil {
		log.Fatal(err)
	}
	db := NewDBWorker(client, database)
	once.Do(func() {
		db.Clear()
		if err := db.CreateIndex(context.Background()); err != nil {
			panic(fmt.Sprintf("create index failed with err:%s", err))
		}
	})

	defer db.client.Disconnect(context.Background())
	b.ResetTimer()
	for i := 0; i < b.N; i++ {
		b.StopTimer()
		worker := generateAWorker()
		b.StartTimer()
		if _, err := db.InsertWorker(context.Background(), worker); err != nil {
			panic(fmt.Sprintf("insert worker failed with err:%s", err))
		}
	}
}

func BenchmarkDbWorker_InsertWorker_Parallel(b *testing.B) {
	client, err := ConnectMongoDB()
	if err != nil {
		log.Fatal(err)
	}
	db := NewDBWorker(client, database)
	once.Do(func() {
		db.Clear()
		if err := db.CreateIndex(context.Background()); err != nil {
			panic(fmt.Sprintf("create index failed with err:%s", err))
		}
	})
	defer db.client.Disconnect(context.Background())
	b.RunParallel(func(pb *testing.PB) {
		for pb.Next() {
			worker := generateAWorker()
			if _, err := db.InsertWorker(context.Background(), worker); err != nil {
				panic(fmt.Sprintf("insert worker failed with err:%s", err))
			}
		}
	})
}

func BenchmarkDbWorker_UpdateHardware(b *testing.B) {
	client, err := ConnectMongoDB()
	if err != nil {
		log.Fatal(err)
	}
	b.StopTimer()
	db := NewDBWorker(client, database)

	defer db.client.Disconnect(context.Background())
	once.Do(func() {
		db.Clear()
		if err := db.CreateIndex(context.Background()); err != nil {
			panic(fmt.Sprintf("create index failed with err:%s", err))
		}
		idlist = initdata(client, 10000, false, false, true)
	})
	b.StartTimer()

	b.ResetTimer()
	for i := 0; i < b.N; i++ {
		b.StopTimer()

		idx := rand.Intn(len(idlist))
		nhardware := generateAHardware()

		b.StartTimer()
		if err := db.UpdateHardware(context.Background(), idlist[idx], nhardware); err != nil {
			panic(fmt.Sprintf("update worker failed with err:%s", err))
		}
	}
}

func BenchmarkDbWorker_UpdateHardware_Parallel(b *testing.B) {
	client, err := ConnectMongoDB()
	if err != nil {
		log.Fatal(err)
	}
	b.StopTimer()
	db := NewDBWorker(client, database)
	defer db.client.Disconnect(context.Background())
	once.Do(func() {
		db.Clear()
		if err := db.CreateIndex(context.Background()); err != nil {
			panic(fmt.Sprintf("create index failed with err:%s", err))
		}
		idlist = initdata(client, 10000, false, false, true)
	})

	b.StartTimer()
	b.ResetTimer()

	b.RunParallel(func(pb *testing.PB) {
		for pb.Next() {
			b.StopTimer()
			idx := rand.Intn(len(idlist))
			nhardware := generateAHardware()
			b.StartTimer()
			if err := db.UpdateHardware(context.Background(), idlist[idx], nhardware); err != nil {
				panic(fmt.Sprintf("update worker failed with err:%s", err))
			}
		}
	})
}

func BenchmarkDbWorker_UpdateGPUUsage_Parallel(b *testing.B) {
	client, err := ConnectMongoDB()
	if err != nil {
		log.Fatal(err)
	}
	b.StopTimer()
	db := NewDBWorker(client, database)
	defer db.client.Disconnect(context.Background())
	once.Do(func() {
		db.Clear()
		if err := db.CreateIndex(context.Background()); err != nil {
			panic(fmt.Sprintf("create index failed with err:%s", err))
		}
		idlist = initdata(client, 10000, false, false, true)
	})
	b.StartTimer()
	b.ResetTimer()
	b.RunParallel(func(pb *testing.PB) {
		for pb.Next() {
			b.StopTimer()
			idx := rand.Intn(len(idlist))
			usage := rand.Intn(10) + 50
			memfree := int64(rand.Intn(10) + 10)
			pwoer := rand.Intn(10) + 20
			temp := rand.Intn(10) + 30
			gpuusages := []types.GpuUsage{
				{0, usage, memfree, pwoer, temp},
				{1, usage + 1, memfree + 1, pwoer + 1, temp + 1},
				{2, usage + 2, memfree + 2, pwoer + 2, temp + 2},
			}
			b.StartTimer()
			if err := db.UpdateGPUUsage(context.Background(), idlist[idx], gpuusages); err != nil {
				panic(fmt.Sprintf("update gpu usage failed with err:%s", err))
			}
		}
	})

}

func BenchmarkDbWorker_FindWorkerByInstallModelAndSortByGpuRam(b *testing.B) {
	client, err := ConnectMongoDB()
	if err != nil {
		log.Fatal(err)
	}
	b.StopTimer()
	db := NewDBWorker(client, database)
	defer db.client.Disconnect(context.Background())

	once.Do(func() {
		db.Clear()
		if err := db.CreateIndex(context.Background()); err != nil {
			panic(fmt.Sprintf("create index failed with err:%s", err))
		}
		idlist = initdata(client, 10000, false, false, true)
	})
	b.StartTimer()
	b.ResetTimer()
	for i := 0; i < b.N; i++ {
		b.StopTimer()
		installedModelId := getRandId(maxModelId)
		performance := generateAGpuPerformance()
		ram := int64(100 * 1024) //generateAGpuRam()
		b.StartTimer()
		if w, err := db.FindWorkerByInstallModelAndSortByGpuRam(context.Background(), installedModelId, performance, ram, 10); err != nil {
			panic(fmt.Sprintf("find worker failed with err:%s", err))
		} else if len(w) == 0 {
			//b.Logf("FindWorkerByInstallModelAndSortByGpuRam find %d with id %s\n", len(w), installedModelId)
		}
	}
}

func BenchmarkDbWorker_FindWorkerByInstallModelAndSortByGpuRam_Parallel(b *testing.B) {
	client, err := ConnectMongoDB()
	if err != nil {
		log.Fatal(err)
	}
	b.StopTimer()
	db := NewDBWorker(client, database)
	defer db.client.Disconnect(context.Background())

	once.Do(func() {
		db.Clear()
		if err := db.CreateIndex(context.Background()); err != nil {
			panic(fmt.Sprintf("create index failed with err:%s", err))
		}
		idlist = initdata(client, 10000, false, false, true)
	})
	b.StartTimer()
	b.ResetTimer()
	b.RunParallel(func(pb *testing.PB) {
		for pb.Next() {
			b.StopTimer()
			installedModelId := getRandId(maxModelId)
			performance := generateAGpuPerformance()
			ram := int64(100 * 1024) //generateAGpuRam()
			b.StartTimer()
			if w, err := db.FindWorkerByInstallModelAndSortByGpuRam(context.Background(), installedModelId, performance, ram, 10); err != nil {
				panic(fmt.Sprintf("find worker failed with err:%s", err))
			} else if len(w) == 0 {
				//b.Logf("FindWorkerByInstallModelAndSortByGpuRam find %d with id %s\n", len(w), installedModelId)
			} else {

			}
		}
	})
}

func BenchmarkDbWorker_FindWorkerByRunningModelAndSortByWaitTime(b *testing.B) {
	client, err := ConnectMongoDB()
	if err != nil {
		log.Fatal(err)
	}
	b.StopTimer()
	db := NewDBWorker(client, database)

	defer db.client.Disconnect(context.Background())
	once.Do(func() {
		db.Clear()
		if err := db.CreateIndex(context.Background()); err != nil {
			panic(fmt.Sprintf("create index failed with err:%s", err))
		}
		idlist = initdata(client, 1000, false, false, true)
	})
	b.StartTimer()
	b.ResetTimer()
	for i := 0; i < b.N; i++ {
		b.StopTimer()
		runningModelId := getRandId(maxModelId)
		b.StartTimer()
		if w, err := db.FindWorkerByRunningModelAndSortByWaitTime(context.Background(), runningModelId, 10); err != nil {
			panic(fmt.Sprintf("find worker failed with err:%s", err))
		} else if len(w) == 0 {
			b.Logf("FindWorkerByRunningModelAndSortByWaitTime find %d with id %s\n", len(w), runningModelId)
		}
	}
}

func BenchmarkDbWorker_FindWorkerByRunningModelAndSortByWaitTime_Parallel(b *testing.B) {
	client, err := ConnectMongoDB()
	if err != nil {
		log.Fatal(err)
	}
	b.StopTimer()
	db := NewDBWorker(client, database)
	defer db.client.Disconnect(context.Background())

	once.Do(func() {
		db.Clear()
		if err := db.CreateIndex(context.Background()); err != nil {
			panic(fmt.Sprintf("create index failed with err:%s", err))
		}
		idlist = initdata(client, 1000, false, false, true)
	})
	b.StartTimer()
	b.ResetTimer()
	b.RunParallel(func(pb *testing.PB) {
		for pb.Next() {
			b.StopTimer()
			runningModelId := getRandId(maxModelId)
			b.StartTimer()
			if w, err := db.FindWorkerByRunningModelAndSortByWaitTime(context.Background(), runningModelId, 10); err != nil {
				panic(fmt.Sprintf("find worker failed with err:%s", err))
			} else if len(w) == 0 {
				b.Logf("FindWorkerByRunningModelAndSortByWaitTime find %d with id %s\n", len(w), runningModelId)
			}
		}
	})
}
